/*
 *
 * BEGIN HEADER
 *
 * ---- 
 *
 * $ID: PantherProxy.java,v 1.6 2001/06/12 20:58:43 burton Exp $
 * $Project: http://panther.openprivacy.org $
 * $CVSROOT: :pserver:anoncvs@sierra.openprivacy.org:/usr/local/cvs/public $
 * $WebCVS: http://www.openprivacy.org/cgi-bin/cvsweb/cvsweb.cgi/panther/ $
 * $Mailing-List: http://www.openprivacy.org/lists/ $
 * $Bugzilla: http://bugzilla.openprivacy.org/ $
 * Copyright 2001 OpenPrivacy.org.  All rights reserved.
 *
 * ---- 
 *
 * Copyright 2001 OpenPrivacy.org.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the LICENSE which you should have obtaind with this package. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.
 *
 * END HEADER
 * 
 */

package org.openprivacy.panther.implementations;

import java.io.*;
import java.net.*;
import java.util.*;

import talon.*;
import talon.resources.*;
import talon.util.net.*;

import org.openprivacy.panther.components.*;
import org.openprivacy.panther.implementations.xml.*;
import org.openprivacy.panther.util.*;

/**
 * 
 * The UpdateManager handles updating the Cache with URLs which are located
 * remotely.  You are not gauranteed that it will return any data right away,
 * instead it might perform some operation asynchronously and give you an
 * Exception.
 *
 * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
 * @version $Id: UpdateManager.java,v 1.7 2001/08/09 23:52:46 burton Exp $
 */
public class UpdateManager {

    /**
     * 
     * <p>
     * This holds the number of miliseconds that a URL *must* remain in the
     * cache before it is allowed to be updated again.
     *
     * <p>
     * Basically this value is compared to the lastModified date for the
     * ProxyEntry, if the result is less than this value, a cache update is
     * aborted.  This is used as defensive programming against a errant thread
     * trying to update the cache too often.  It is also used so that multiple
     * thread, each unaware of each other, can request cache updates around the
     * same time and only one will complete and the other will benefit from the
     * recently requested data.
     *
     * <p>
     * Internally this is represented as minutes * seconds 1000 for code
     * clarity purposes.
     *
     */
    public static final long DEFAULT_MIN_CACHED_INTERVAL = 10 * 60 * 1000; //ten minutes 
    
    private static UpdateManager instance = null;

    private Hashtable lastAttempts = new Hashtable();
    
    /**
     * Update the cache if necessary.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: UpdateManager.java,v 1.7 2001/08/09 23:52:46 burton Exp $
     */
    public synchronized void update( String location ) throws IOException {

        if ( this.stale( location ) ) {

            setLastUpdateAttempt( location );
            
            new UpdateThread( location ).start();

        } 
        
    }

    
    /**
     * <p>
     * Last time (in miliseconds) which the Proxy tried to update this URL.
     *
     * <p>
     * This is kept so that if an update fails, we can at least keep track
     * of the attempts so that we don't keep requesting a URL that is dead
     * (can't route to it, invalid URL, etc)
     * 
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: UpdateManager.java,v 1.7 2001/08/09 23:52:46 burton Exp $
     */
    public long getLastUpdateAttempt( String location ) {

        if ( lastAttempts.containsKey( location ) ) {

            Long l = (Long)lastAttempts.get( location );

            return l.longValue();
            
        } else {

            return -1;

        }

    }

    /**
     * 
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: UpdateManager.java,v 1.7 2001/08/09 23:52:46 burton Exp $
     */
    public void setLastUpdateAttempt( String location ) {

        lastAttempts.put( location, new Long( System.currentTimeMillis() ) );
        
    }
    
    /**
     * Return true if this URL needs to be updated.  
     *
     * @see UpdateManager#REQUIRED_CACHED_PRIOR_UPDATE_MILI
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: UpdateManager.java,v 1.7 2001/08/09 23:52:46 burton Exp $
     */
    public boolean stale( String location ) {

        long delta = 0;

        long correctInterval = 0;

        boolean contains = false; //true if the proxy contains this URL.
        
        //FIXME: if the cach does NOT contain the URL... it is obviously stale.
        try {
             
            ProxyEntry entry = ProxyEntryManager.getInstance().get( location );

            correctInterval = entry.getMinCachedInterval();
            
            contains = true;
            
            //use the most recent value for the delta.
            if ( entry.getLastModified() > getLastUpdateAttempt( location )  ) {

                delta = System.currentTimeMillis() - entry.getLastModified();

            } else {

                delta = System.currentTimeMillis() - getLastUpdateAttempt( location );

            }

        } catch ( IOException e ) {

            delta = System.currentTimeMillis() - getLastUpdateAttempt( location );

            correctInterval = DEFAULT_MIN_CACHED_INTERVAL;

        }


        //determine the correct interval to use for determining if this is tale.
        
        //NOTE: this is what determines if we are stale.
        //
        // FIXME: we shouldn't update the URL just because getLastUpdateAttempt
        // is -1 (IE never been updated for this JVM instance).  Essentially
        // this was left as a developer feature since this means we get fresh
        // content when we start up panther.  The only problem is that this hits
        // webserver a lot.  Consider fixing this in the future.
        //
        if ( delta > correctInterval || getLastUpdateAttempt( location ) == -1 ) {
            
            return true;
            
        } else {
            
            return false;
            
        }

    }
    
    /**
     * Get this as a singleton.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: UpdateManager.java,v 1.7 2001/08/09 23:52:46 burton Exp $
     */
    public static UpdateManager getInstance() {

        if ( instance == null ) { 

            instance = new UpdateManager();

        }

        return instance;
    }


}

